# -*- coding: utf-8 -*-
from .barbados import Barbados
from .brazil import (
    Brazil, BrazilAcre, BrazilAlagoas, BrazilAmapa, BrazilAmazonas,
    BrazilBahia, BrazilCeara, BrazilDistritoFederal, BrazilEspiritoSanto,
    BrazilGoias, BrazilMaranhao, BrazilMatoGrosso, BrazilMatoGrossoDoSul,
    BrazilPara, BrazilParaiba, BrazilPernambuco, BrazilPiaui,
    BrazilRioDeJaneiro, BrazilRioGrandeDoNorte, BrazilRioGrandeDoSul,
    BrazilRondonia, BrazilRoraima, BrazilSantaCatarina, BrazilSaoPauloState,
    BrazilSaoPauloCity, BrazilSergipe, BrazilTocantins, BrazilVitoriaCity,
    BrazilVilaVelhaCity, BrazilCariacicaCity, BrazilGuarapariCity,
    BrazilSerraCity, BrazilBankCalendar
)
from .canada import (
    Canada, Ontario, Quebec, BritishColumbia, Alberta, Saskatchewan, Manitoba,
    NewBrunswick, NovaScotia, PrinceEdwardIsland, Newfoundland, Yukon,
    NorthwestTerritories, Nunavut
)
from .chile import Chile
from .colombia import Colombia
from .mexico import Mexico
from .panama import Panama
from .paraguay import Paraguay


__all__ = (
    # Brazil & its states.
    'Brazil',
    'BrazilAcre',
    'BrazilAlagoas',
    'BrazilAmapa',
    'BrazilAmazonas',
    'BrazilBahia',
    'BrazilCeara',
    'BrazilDistritoFederal',
    'BrazilEspiritoSanto',
    'BrazilGoias',
    'BrazilMaranhao',
    'BrazilMatoGrosso',
    'BrazilMatoGrossoDoSul',
    'BrazilPara',
    'BrazilParaiba',
    'BrazilPernambuco',
    'BrazilPiaui',
    'BrazilRioDeJaneiro',
    'BrazilRioGrandeDoNorte',
    'BrazilRioGrandeDoSul',
    'BrazilRondonia',
    'BrazilRoraima',
    'BrazilSantaCatarina',
    'BrazilSaoPauloState',
    'BrazilSaoPauloCity',
    'BrazilSergipe',
    'BrazilTocantins',
    'BrazilVitoriaCity',
    'BrazilVilaVelhaCity',
    'BrazilCariacicaCity',
    'BrazilGuarapariCity',
    'BrazilSerraCity',
    'BrazilBankCalendar',
    # Canada
    'Canada',
    'Ontario',
    'Quebec',
    'BritishColumbia',
    'Alberta',
    'Saskatchewan',
    'Manitoba',
    'NewBrunswick',
    'NovaScotia',
    'PrinceEdwardIsland',
    'Newfoundland',
    'Yukon',
    'NorthwestTerritories',
    'Nunavut',
    # Other american countries
    'Barbados',
    'Chile',
    'Colombia',
    'Mexico',
    'Panama',
    'Paraguay',
)
