# -*- coding: utf-8 -*-
from __future__ import (absolute_import, division, print_function,
                        unicode_literals)

from .core import UnitedStates
from ..registry_tools import iso_register


@iso_register('US-ME')
class Maine(UnitedStates):
    """Maine"""
    include_thanksgiving_friday = True
    include_patriots_day = True
